<?php

namespace Drupal\trinion_base\Controller;

use Drupal\Core\Ajax\AjaxResponse;
use Drupal\Core\Ajax\FocusFirstCommand;
use Drupal\Core\Ajax\HtmlCommand;
use Drupal\Core\Ajax\InvokeCommand;
use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Entity\Element\EntityAutocomplete;
use Drupal\taxonomy\Entity\Term;
use Drupal\user\Entity\User;
use Symfony\Component\HttpFoundation\JsonResponse;

class InlineChangeFieldController extends ControllerBase {

  public function showField($node, $field_name) {
    $nid = $node->id();
    switch ($field_name) {
      case 'field_tz_ikonka':
        $terms = \Drupal::entityTypeManager()->getStorage('taxonomy_term')->loadByProperties(['vid' => 'prioritet_zadachi']);
        $options = [];
        foreach ($terms as $term)
          $options[$term->id()] = $term->label();
        $build = [
          '#type' => 'select',
          '#options' => $options,
          '#value' => $node->get('field_tz_prioritet')->getString(),
        ];
        break;
      case 'field_tz_ikonka_1':
        $terms = \Drupal::entityTypeManager()->getStorage('taxonomy_term')->loadByProperties(['vid' => 'tz_statusy_zadach']);
        $options = [];
        foreach ($terms as $term)
          $options[$term->id()] = $term->label();
        $build = [
          '#type' => 'select',
          '#options' => $options,
          '#value' => $node->get('field_tz_status_zadachi')->getString(),
        ];
        break;
      case 'field_tb_nick_name':
        $resposible = $node->get('field_tz_otvetstvennyy')->first()->entity;
        $build = [
          '#type' => 'textfield',
          '#value' => '',
          '#attributes' => ['data-autocomplete-path' => "/inline-autocomplete-field/{$nid}/{$field_name}"],
          '#value' => $resposible->get('field_tb_nick_name')->getString() . ' (' . $resposible->id() . ')',
        ];
        break;
      default:
        $type = $node->get($field_name)->getFieldDefinition()->getType();
        switch ($type) {
          case 'string':
            $build = [
              '#type' => 'textfield',
              '#value' => $node->get($field_name)->getString(),
            ];
            break;
        }
    }

    $resposne = new AjaxResponse();
    $resposne->addCommand(new HtmlCommand("#{$field_name}-{$nid}", $build));
    $resposne->addCommand(new FocusFirstCommand("#{$field_name}-{$nid}"));
    return $resposne;
  }

  public function changeField($node, $field_name) {
    $nid = $node->id();
    $new_value = \Drupal::request()->get('data');
    switch ($field_name) {
      case 'field_tz_ikonka':
        $node->set('field_tz_prioritet', $new_value);
        $node->save();
        $term = Term::load($new_value);
        $build = $term->get('field_tz_ikonka')->getString();
        break;
      case 'field_tz_ikonka_1':
        $node->set('field_tz_status_zadachi', $new_value);
        $node->save();
        $term = Term::load($new_value);
        $build = $term->get('field_tz_ikonka')->getString();
        break;
      case 'field_tb_nick_name':
        switch ($node->bundle()) {
          case 'zadacha':
            $target_field = 'field_tz_otvetstvennyy';
            break;
        }
        if ($target_field && $uid = EntityAutocomplete::extractEntityIdFromAutocompleteInput($new_value)) {
          $node->set($target_field, $uid);
          $node->save();
          $build = $new_value;
        }
        else {
          if ($current = $node->get($target_field)->first()) {
            $build = $current->entity->get('field_tb_nick_name')->getString();
          }
        }
        break;
      default:
        $type = $node->get($field_name)->getFieldDefinition()->getType();
        switch ($type) {
          case 'string':
            $node->set($field_name, $new_value);
            $node->save();
            $build = $new_value;
            break;
        }
    }

    $resposne = new AjaxResponse();
    $resposne->addCommand(new HtmlCommand("#{$field_name}-{$nid}", $build));
    return $resposne;
  }

  public function autocompleteField($node, $field_name) {
    $matches = [];
    $term = \Drupal::request()->get('term');
    switch ($field_name) {
      case 'field_tb_nick_name':
        $query = \Drupal::entityQuery('user')
          ->condition('field_tb_nick_name', $term . '%', 'LIKE');
        $query->range(0, 10);
        $res = $query->accessCheck()->execute();
        foreach (User::loadMultiple($res) as $user) {
          $value = $user->get('field_tb_nick_name')->getString();
          $value .= ' (' . $user->id() . ')';
          $matches[] = [
            'label' => $value,
            'value' => $value,
          ];
        }
        break;
    }
    return new JsonResponse($matches);
  }
}
